<?php

/* session.lib.php
   Defines a class used to manage Siteation sessions.
   Part of Siteation.
   Copyright (C) 2005 Studio 703.  All rights reserved.
*/


require_once("query.lib.php");
require_once("site.lib.php");
require_once("misc.lib.php");
require_once("site.error.php");

// This class handles Siteation sessions
class SITE_SESSION {
  // Stores the site handler used to manage this session
  private $site;
  // Stores the data source used to manage this session
  private $dataSource;
  // Stores the session settings
  private $settings;
  // Indicates whether or not the session information has changed
  private $changed;
  
  // Constructor
  function __construct($siteHandler) {
    // Continue only if we have a valid reference to a site handler
    if (!isset($siteHandler)) return(seInvalidParam);
    // Stash the site handler for future use
    $this->site = $siteHandler;
    // Stash the data source for future use
    $this->dataSource = $this->site->DataSource();
    $this->settings = array();
    DebugOut(get_class($this), array(
      "<strong>__construct()</strong> has started."));
    // Stash the session handlers
    session_set_save_handler(
      array($this,"Begin"),
      array($this,"EndSession"),
      array($this,"Get"),
      array($this,"Put"),
      array($this,"Purge"),
      array($this,"PurgeOldSessions"));
    // Set the session cookie name
    session_name("sid");
    // Begin the new session
    session_start();
  }
  
  function __destruct()
  {
    DebugOut(get_class($this), array(
      "<strong>__destruct()</strong> has started."));
    // Ensure that the session data is flushed
    session_write_close();
  }
  
  // Begins a new or existing session
  function Begin($sessionPath, $sessionName) {
    // Does nothing
    return true;
  }
  
  // Ends an existing session
  function EndSession() {
    // Does nothing
    return true;
  }
  
  // Purges the specified session from the database
  function Purge($sessionID) {
    GLOBAL $SITE_CONFIG;
    // Create a query
    $session = new QUERY($this->dataSource);
    // Set the table on which to operate
    $session->location = array($SITE_CONFIG["TABLE_PREFIX"]."sessions");
    // Set the query criteria
    $session->criteria = array(
      "SessionID = ", Quote($sessionID));
    // Execute the query
    $result = $session->Delete();
    return($result);
  }
  
  // Purges all old sessions in the database
  function PurgeOldSessions() {
    GLOBAL $SITE_CONFIG;
    // If the site is configured to remove old sessions, do so now
    if ($this->site->Setting("Sessions", "RemoveOld") == "Y") {
      // Create a query
      $sessionQuery = new QUERY($this->dataSource);
      // Set the table on which to operate
      $sessionQuery->location = array($SITE_CONFIG["TABLE_PREFIX"]."sessions");
      // Set the query criteria
      $sessionQuery->criteria = array (
        "Expires <=", Quote($this->dataSource->TimeValue()));
      // Execute the query
      $result = $sessionQuery->Delete();
      return($result);
    }
    else return true;
  }
  
  // Retrieves the specified session from the database
  function Get($sessionID) {
    global $SITE_CONFIG;
    // Create a query
    $sessionQuery = new QUERY($this->dataSource);
    // Select the table to search
    $sessionQuery->location = array(
      $SITE_CONFIG["TABLE_PREFIX"]."sessions" => "*");
    // Set the criteria
    $sessionQuery->criteria = array(
      "SessionID=", Quote($sessionID));
    // Run the query
    $result = $sessionQuery->Select();
    // If the query failed, indicate that failure
    if ($result <= 0) {
      // We will be creating a new session for an anonymous site visitor
      $_SESSION["UserName"] = $this->site->Setting("Defaults", "VisitorAccount");
      return("");
    }
    // Retrieve the row containing the session information
    $this->settings = $sessionQuery->NextRowVerbose();
    // Indicate success
    return($this->settings["SessionData"]);
  }
  
  // Flushes session information to the database
  function Put($sessionID, $sessionData) {
    global $SITE_CONFIG;
    DebugOut(get_class($this), array(
      "<strong>Put()</strong> has started."));
    if (empty($this->settings)) return($this->Create($sessionID, $sessionData));
    // Create a query to update the session
    $session = new QUERY($this->dataSource);
    // Set the table on which to operate
    $session->location = array($SITE_CONFIG["TABLE_PREFIX"]."sessions");
    // Store the updated fields and values
    $this->settings["SessionData"] = $sessionData;
    $session->fieldsValues = $this->settings;
    // Set the criteria so that the correct session is updated
    $session->criteria = array(
      "SessionID=",Quote($this->settings["SessionID"]));
    // Run the query
    $result = $session->Update();
    // If the query failed, return the error
    return($result >= 1);
  }
  
  // Creates a new session entry in the database
  function Create($sessionID, $sessionData) {
    global $SITE_CONFIG;
    DebugOut(get_class($this), array(
      "<strong>Create()</strong> has started."));    
    // Create a query to insert the session
    $session = new QUERY($this->dataSource);
    // Set the table on which to operate
    $session->location = array($SITE_CONFIG["TABLE_PREFIX"]."sessions");
    // Store the session data
    $this->settings["SessionID"] = $sessionID;
    $this->settings["SessionData"] = $sessionData;
    // Calculate the session expiration
    $this->Renew();
    // Write the session data
    $session->fieldsValues = $this->settings;
    $result = $session->Insert();
    DebugOut(get_class($this), array(
      "<strong>Create()</strong> has completed with a value of <strong>", $result, "</strong>"));
    return($result);
  }
  
  /* Renews the session based on the session TTL values specified in the
     site configuration */
  function Renew() {
    GLOBAL $SITE_CONFIG;
    /* If the user is an anymous site visitor, extend the session lifetime by
       the value of the VisitorSessionTTL option */
    if (!$this->LoggedIn()) {
      $sessionTTL = $this->site->Setting("Sessions", "VisitorTTL");
    }
    /* Otherwise, extend the session lifetime by the value of the UserSessionTTL
       option */
    else {
      // Create a persistent login 
      $sessionTTL = $this->site->Setting("Sessions", "UserTTL");
    }
    /* Convert the session TTL to seconds, since the site options specify it
       in minutes */
    $sessionTTL *= 60;
    // Add the current time to it to cast it into the future
    $sessionTTL += time();
    // Update the session's expiration timestamp
    $this->settings["Expiration"] = $this->dataSource->TimeValue($sessionTTL);
    return true;
  }
  
  /* Determines whether or not this session is for a logged-in user or a
     visitor, based on the value of the USER_NAME session variable */
  function LoggedIn() {
    // If we don't know who the user is, they can't very well be logged in...
    if (!isset($_SESSION["UserName"]) || empty($_SESSION["UserName"])) return(FALSE);
    /* Test the user name against the name of the user account used for site
       visitors */
    return($_SESSION["UserName"] !=
      $this->site->Setting("Defaults", "VisitorAccount"));
  }
}

?>
