<?php

/* site.lib.php
   Defines a class used to interface with a Siteation site.
   Part of Siteation.
   Copyright (C) 2005 Studio 703.  All rights reserved.
   
*/

require_once("site.error.php");
require_once("misc.lib.php");

// This class interfaces with the Siteation site.
class SITE {
  // Stores the data source object used to access the site
  private $dataSource;
  // Stores site settings read from the database
  private $settings;
  // Tracks changes to the site settings
  private $changes;
  
  // Constructor
  function __construct() {
    // Determine which data source we will be using
    global $SITE_CONFIG;
    switch ($SITE_CONFIG["SQL_SERVER"]) {
      // We are using MySQL
      case "MYSQL":
        // Include the appropriate library
        require_once("mysql.lib.php");
        // Create the data source instance
        $this->dataSource = new MySQLDataSource(
          $SITE_CONFIG["SQL_HOSTNAME"],
          $SITE_CONFIG["SITE_DATABASE"]);
        break;
      // We are using Microsoft SQL server
      case "MSSQL":
        // Include the appropriate library
        require_once("mssql.lib.php");
        // Create the data source instance
        $this->dataSource = new MSSQLDataSource(
          $SITE_CONFIG["SQL_HOSTNAME"],
          $SITE_CONFIG["SITE_DATABASE"]);
          break;
      // We are using SQLite
      case "SQLITE":
        // Include the appropriate library
        require_once("sqlite.lib.php");
        // Create the data source instance
        $this->dataSource = new SQLiteDataSource(
          $SITE_CONFIG["SQL_HOSTNAME"],
          $SITE_CONFIG["SITE_DATABASE"]);
          break;
    }
    // If we failed to open a connection to the data source, indicate an error
    if (!isset($this->dataSource))
      DiePrettily(seDataSource, array(
        "Hostname" => $SITE_CONFIG["SQL_HOSTNAME"],
        "Database" => $SITE_CONFIG["SITE_DATABASE"]));
    // Try to open the database
    $result = $this->dataSource->Connect(
      $SITE_CONFIG["SQL_USER_NAME"],
      $SITE_CONFIG["SQL_PASSWORD"]);
    // If we failed to open the database, indicate an error
    if ($result < 0)
      DiePrettily(seDataSource, array(
        "Hostname" => $SITE_CONFIG["SQL_HOSTNAME"],
        "Database" => $SITE_CONFIG["SITE_DATABASE"]));
  }
  
  // Destructor
  function __destruct() {
    // Flush any unsaved changes to the site settings
    $this->FlushChanges();
  }
  
  // Retrieves the data source handler used by this site handler
  function DataSource() {
    // Return the data source handler
    return($this->dataSource);
  }
  
  // Retrieves site configuration information from the database
  function Get() {
    global $SITE_CONFIG;
    // Construct a query to retrieve the site settings
    $site = new QUERY($this->dataSource);
    // Set the name of the table to be searched
    $site->location = array($SITE_CONFIG["TABLE_PREFIX"]."config" => 
      array("GroupName", "KeyName", "KeyValue"));
    $site->criteria = "ClassName='Site'";
    // Execute the query
    $result = $site->Select();
    // If we failed to retrieve site options, indicate an error
    if ($result <= 0) return(seQuery);
    // Retrieve the first row of settings
    $resultRow = $site->NextRowVerbose();
    // Retrieve all rows and store them
    while($resultRow) {
      // Get the key group and its associated values
      $groupName = $resultRow["GroupName"];
      $keyName = $resultRow["KeyName"];
      $keyValue = $resultRow["KeyValue"];
      // Store the value
      $this->settings[$groupName][$keyName] = $keyValue;
      // Retrieve the next row
      $resultRow = $site->NextRowVerbose();
    }
    // Free the query
    $site->__destruct();
    // Reset the list of changes
    $this->changes = NULL;
    // Return success
    return(seOkay);
  }
  
  // Flushes site settings changes to the database
  function FlushChanges() {
    // Continue only if there have actually been changes
    if (empty($this->changes)) return(seOkay);
    global $SITE_CONFIG;
    // Construct a query
    $site = new QUERY($this->dataSource);
    // Set the table to be updated
    $site->location = array($SITE_CONFIG["TABLE_PREFIX"]."config");
    // Run through each change and update the database
    foreach($this->changes as $changedGroup => $changedKeys) {
      foreach($changedKeys as $changedKey) {
       // Set the record to be updated
        $site->criteria = array(
          "ClassName='Site'", " AND ",
          "GroupName=", Quote($changedGroup), " AND ",
          "KeyName=", Quote($changedKey));
        // Store the new values for the record
        $site->fieldsValues = array(
          "KeyValue" => $this->settings[$changedGroup][$changedKey]);
        // Execute the query
        $result = $site->Update();
        // If the query failed, signal an error
        if ($result < 0) return($result);
      }
    }
    // Reset the list of changes
    $this->changes = NULL;
    return(seOkay);
  }
  
  /* Retrieves all site settings.  Optionally, you can specify that the site
     settings should be loaded from the database first. */
  function Settings($refresh = FALSE) {
    // Determine whether we need to retrieve the options from the database
    if (empty($this->settings) || $refresh) {
      // Retrieve the settings from the database
      $result = $this->Get();
      // If, for some reason, we failed, return the error
      if ($result < 0) return ($result);
    }
    // Return the settings
    return($this->settings);
  }
  
  /* Sets all settings at once.  Optionally, you can specify that the changes
     should be written immediately to the database. */
  function SetAll($settings, $updateNow = FALSE) {
    // Store the new settings
    $this->settings = $settings;
    // Mark the changes
    foreach ($setting as $keyValue => $ignored)
      $this->changes[$keyValue] = TRUE;
    // Flush the changes immediately, if necessary
    if ($updateNow) return($this->FlushChanges());
    // Return success
    return(seOkay);
  }
  
  /* Retrieves a single site setting.  Optionally, you can specify that the site
     settings should be loaded from the database first. */
  function Setting($keyGroup, $keyName, $refresh = FALSE) {
    // Continue only if we have a valid-looking key group and key name
    if (empty($keyGroup) || empty($keyName)) return(seInvalidParam);
    // Determine if we need to retrieve the settings from the database
    if (empty($this->settings) || $refresh) {
      // Retrieve the settings from the database
      $result = $this->Get();
      // If, for some reason, we failed, return the error
      if ($result < 0) return(seQuery);
    }
    // Retrieve the setting, if it exists
    if (isset($this->settings[$keyGroup][$keyName]))
      return($this->settings[$keyGroup][$keyName]);
    else return(NULL);
  }
  
  /* Sets a single site setting.  Optionally, you can specify that the change
     should be immediately flushed to the database. */
  function Set($keyGroup, $keyName, $keyValue, $updateNow = FALSE) {
    // Continue only if we are working with valid-looking information
    if (empty($keyGroup) || empty($keyName) ||
      !isset($this->settings[$keyGroup])) return(seInvalidParam);
    // Store the new setting
    $this->settings[$keyGroup][$keyName] = $keyValue;
    // Store the change
    $this->changes[$keyGroup][] = $keyName;
    // Update immediately, if necessary
    if ($updateNow) return($this->FlushChanges());
    return(seOkay);
  }
  
  /* Determines whether or not the specified channel exists.  If it does, the
     primary key of the channel is returned. */
  function ChannelExists($channelName) {
    // Continue only if we have valid information
    if (empty($channelName)) return(seInvalidParam);
    global $SITE_CONFIG;
    // Create a query
    $channel = new Query($this->dataSource);
    // Select the table to be searched and the field to be returned
    $channel->location = array(
      $SITE_CONFIG["TABLE_CHANNELS"] => "ID");
    // Set the criteria
    $channel->criteria = array(
      "ChannelName" => "=".Quote($channelName));
    // Run the query
    $result = $channel->Select();
    // If the query failed, assume that the channel does not exist
    if ($result <= 0) return(seResourceNotFound);
    // Retrieve the row that contains the channel ID
    $resultRow = $channel->NextRowVerbose();
    // Free the query
    $channel->__destruct();
    // Return the result
    $return($resultRow["ID"]);
  }
  
  /* Retrieves the name of every channel on the site */
  function ChannelNames($filter="@ALL") {
    global $SITE_CONFIG;
    // Create the array that holds our channel names
    $channelList = array();
    // Create a query
    $channelNames = new Query($this->dataSource);
    // Select the table to be searched and the fields to be returned
    $channelNames->location = array(
      $SITE_CONFIG["TABLE_CHANNELS"] => array("ID", "ChannelName", "Parent", "Icon"));
    // Set the order in which to retrieve channel names
    $channelNames->sort = array(
      "ASC" => array("ID", "ChannelName"));
    // Filter the list for public channels, if specified
    if ($filter == "@PUBLIC") $channelNames->criteria = array(
      "[Public]" => "= 'Y'");
    // Filter the list for private channels, if specified
    else if ($filter == "@PRIVATE") $channelNames->criteria = array(
      "[Public]" => "= 'N'");
    // Run the query
    $result = $channelNames->Select();
    // If the query failed, return an error
    if ($result <= 0) return(seDataUnavailable);
    // Retrieve each row from the query
    $resultRow = $channelNames->NextRowVerbose();
    while ($resultRow) {
      // Store the channel name
      $channelList[] = array(
        "Parent" => $resultRow["Parent"],
        "ChannelName" => $resultRow["ChannelName"],
        "Icon" => $resultRow["Icon"]);
      // Get the next row
      $resultRow = $channelNames->NextRowVerbose();
    }
    // Free the query
    $channelNames->__destruct();
    // Return the list of channels
    return($channelList);
  }
  
  /* Determines if the specified user name exists */
  function UserExists($userName) {
    global $SITE_CONFIG;
    // Create a query
    $users = new Query($this->dataSource);
    // Select the table to be searched and the fields to be returned
    $users->location = array(
      $SITE_CONFIG["TABLE_USERS"] => array("ID", "UserName"));
    // Set the query criteria
    $users->criteria = array(
      "UserName" => "=".$userName );
    // Run the query
    $result = $users->Select();
    // If the query failed, return false
    if ($result <= 0) return(0);
    // Retrieve the matching user
    $resultRow = $users->NextRowVerbose();
    // Return the matching user ID
    return($resultRow["ID"]);
  }
  
  // Deletes the record for the specified user
  function DeleteUser($userName) {
    // If no user name was specified, exit
    if (empty($userName)) return(seInvalidParam);
    global $SITE_CONFIG;
    // Create a query
    $user = new QUERY($this->dataSource);
    // Select the table to update
    $user->location = $SITE_CONFIG["TABLE_USERS"];
    // Stash the query criteria
    $user->criteria = array(
      "UserName" => "=".$userName);
    // Go!
    $user->Delete();
    return(seOkay);
  }
  
  /* Retrieves site statistics */
  function Statistics() {
    global $SITE_CONFIG;
    // This array holds our output
    $output = array();
    // Retrieve the total number of site views
    $output["Total Number of Hits"] = $this->settings["Statistics"]["ViewCount"];
    // Retrieve the total time spent rendering views
    $output["Total Render Time"] = $this->settings["Statistics"]["TotalRenderTime"].Plural(" second", $this->settings["Stats"]["TotalRenderTime"]);
    // Retrieve the average time taken to render a view
    $output["Average Render Time per View"] = $this->settings["Statistics"]["AvgRenderTime"].Plural(" second", $this->settings["Stats"]["AvgRenderTime"]);
    // Determine the number of unique visitors to the site
    $query = new Query($this->dataSource);
    $result = $query->DirectQuery("SELECT DISTINCT IP FROM ".$SITE_CONFIG["TABLE_SESSIONS"]);
    if ($result >= 0) $output["Number of Unique Visitors"] = $query->RowCount();
    // Determine the number of registed users for the site
    $result = $query->DirectQuery("SELECT ID FROM ".$SITE_CONFIG["TABLE_USERS"]);
    if ($result >= 0) $output["Number of Registered Users"] = $query->RowCount();
    // Determine the number of posts for the site
    $result = $query->DirectQuery("SELECT ID FROM ".$SITE_CONFIG["TABLE_POSTS"]);
    if ($result >= 0) $output["Total Number of Posts"] = $query->RowCount();
    return($output);
  }
  
  /* Retrieves the nth most recently updated posts */
  function RecentUpdates($max) {
    global $SITE_CONFIG;
    // This array holds our output
    $output = array();
    // Construct the query
    $updates = new Query($this->dataSource);
    // Select the table to search
    $updates->location = array(
      $SITE_CONFIG["TABLE_POSTS"] => array("ID", "ChannelName", "Title", "PubDate", "Parent"));
    // Set the criteria
    $updates->criteria = array(
      "Parent" => "= 0");
    // Set the sort order
    $updates->sort = array(
      "DESC" => "PubDate");
    // Run the query
    $result = $updates->Select();
    // If the query succeeded, retrieve the rows returned
    if ($result > 0) {
      $resultRow = $updates->NextRowVerbose();
      while(($resultRow) && (count($output) <= $max)) {
        $output[] = $resultRow;
        $resultRow = $updates->NextRowVerbose();
      }
    }
    // Reverse the output
    $output = array_reverse($output);
    // Return the rows
    return($output);
  }
}
?>
